/*
 * DistanceComponent.java
 *
 * Copyright (c) 2006 Computer-Aided Integrated Systems Laboratory, 
 * St. Petersburg Institute for Informatics and Automation RAS http://cais.lisa.iias.spb.su
 *
 * This software is open-source under the BSD license; see either "license.txt" 
 * or http://jung.sourceforge.net/license.txt for a description.
 */

package connectedsubgraph;

import matching.UnweightedShortestPathUtil;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
//import java.util.HashSet;

import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.decorators.Indexer;

/** Calculates and stores the shortest distance between 
 * connected components within the graph
 */
public class DistanceComponent {
    
    /** vertices of the first connected component */
    Set<Vertex> cc1;
    //List<DistanceComponent> cc1_components;
    //Map<Set<Vertex>, List<DistanceComponent>> cc_neighbours;
    
    /** vertices of the second connected component */
    Set<Vertex> cc2;
    
    /** The shortest path between components 'cc1' and 'cc2'.
     * It does not contain any vertices from 'cc1' and 'cc2'.
     */
    Vertex[]    path;
    
    /** Distance of the shortest path between components 'cc1' and 'cc2' */
    //int         dist;
    
    
    /** Updates sets, updates shortest path between sets.
     */
    public void update(Set<Vertex> set1,Set<Vertex> set2,Vertex[] _path)
    {
        cc1 = set1;
        cc2 = set2;
        path = _path;
        //dist = _path.length - 1;
    }
    
    /** Calculates shortest paths between connected components cc in graph g,
     * creates and returns DistanceComponent list,
     * stores data to the DistanceComponent's 'dist' and 'path'.
     */
    static public List<DistanceComponent> calcShortestDistances(
                    List<Set<Vertex>> cc,UnweightedShortestPath usp)
                    // Graph g, 
    {
        //Indexer id = Indexer.getIndexer(g);
        //UnweightedShortestPath usp = new UnweightedShortestPath(g);
        
        List<DistanceComponent> dc_list = new ArrayList<DistanceComponent>();
        
        for(int i=0;i<cc.size();i++) {
            for(int j=0;j<i;j++) {
                Set<Vertex> set1 = cc.get(i);
                Set<Vertex> set2 = cc.get(j);
                Vertex[] path = UnweightedShortestPathUtil.getShortestPath(set1, set2, usp);
                if(null != path && 0 < path.length) {
                    DistanceComponent dc = new DistanceComponent();
                    dc.update(set1, set2, path);
                    dc_list.add(dc);
                }
            }   
        }
        return dc_list;
    }
    
    
    /** Creates map from connected sets of vertices to neighbours sets via DistanceComponent.
     */
    static public Map<Set<Vertex>, DistanceComponent[]>
            mapSetToNeighbourComponents(List<DistanceComponent> dc_list,List<Set<Vertex>> cc) 
    {
        if(null == dc_list || null == cc)
            return null;
                
        Map<Set<Vertex>, DistanceComponent[]> m = new HashMap<Set<Vertex>, DistanceComponent[]>();
        
        for(Set<Vertex> set1:cc) {
            List<DistanceComponent> dc_new = new ArrayList<DistanceComponent>();
            
            for (DistanceComponent dc:dc_list) {
                if((set1 == dc.cc1 || set1 == dc.cc2) && !dc_new.contains(dc)) {
                    dc_new.add(dc);
                }
            }
            
            m.put(set1, (DistanceComponent[])dc_new.toArray(new DistanceComponent[0]));
        }
        return m;
    }
    
}


/*
 Calculates shortest path and distance between connected components 
    public void calc(Set<Vertex> _cc1,Set<Vertex> _cc2,Graph g) {
        cc1 = _cc1;
        cc2 = _cc2;
        
        // get shortest path from set to set
        // see 
        // UnweightedShortestPathUtil
        // public static Set<Vertex> getShortestPath (Set<Vertex> source,Set<Vertex> dest,UnweightedShortestPath usp) {
        
        
    }
*/
    