/*
 * Copyright (c) 2006 Computer-Aided Integrated Systems Laboratory, 
 * St. Petersburg Institute for Informatics and Automation RAS http://cais.lisa.iias.spb.su
 *
 *
 * This software is open-source under the BSD license; see either
 * "license.txt" or http://jung.sourceforge.net/license.txt for a description.
*/

package matching;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.List;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import edu.uci.ics.jung.algorithms.importance.*;

import edu.uci.ics.jung.utils.MutableDouble;
import edu.uci.ics.jung.utils.UserData;

import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Element;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.ArchetypeVertex;
import edu.uci.ics.jung.graph.decorators.DefaultToolTipFunction;
import edu.uci.ics.jung.graph.decorators.EdgeShape;
import edu.uci.ics.jung.graph.decorators.PickableEdgePaintFunction;
import edu.uci.ics.jung.graph.decorators.EllipseVertexShapeFunction;
import edu.uci.ics.jung.graph.decorators.PickableVertexPaintFunction;
import edu.uci.ics.jung.graph.decorators.VertexIconAndShapeFunction;
import edu.uci.ics.jung.graph.decorators.VertexStringer;
import edu.uci.ics.jung.graph.impl.UndirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.DirectedSparseVertex;
import edu.uci.ics.jung.visualization.DefaultGraphLabelRenderer;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.PluggableRenderer;
import edu.uci.ics.jung.visualization.ShapePickSupport;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;


/** Connect vertices with similar names (in different graphs) by edges.
 *
 * Purpose: debug by visualization.
 */
public class SimVertConnectByEdge {
    
    
    public SimVertConnectByEdge () {
    }
    
    /** Calculates initial similarity of node names (NAME_KEY) via Levenshtein 
     * distance between nodes of sub_graph1 and sub_graph2, stores similarity value 
     * to edge field (SIM_KEY).
     * 
     * The graphs sub_graph1 and sub_graph2 could be discerned due to vertex datum NAME_KEY.
     */
    //public void calcNodesStringSimilarity(Graph graph, Graph sub_graph1, Graph sub_graph2, LevenshteinDistance lev) {
    public void calcNodesStringSimilarity(Graph graph, LevenshteinDistance lev) {
        for(Object o1:graph.getVertices()) {
            Vertex v1 = (Vertex)o1;
            String name1 = (String)v1.getUserDatum(GraphComparator.NAME_KEY);
            boolean b1 = v1.containsUserDatumKey(GraphComparator.SUBGRAPH_N);
            
            for(Object o2:graph.getVertices()) {
                Vertex v2 = (Vertex)o2;
                String name2 = (String)v2.getUserDatum(GraphComparator.NAME_KEY);

                boolean b2 = v2.containsUserDatumKey(GraphComparator.SUBGRAPH_N);
                // System.out.println("b1:"+b1+" b2:"+b2);
                if(b1 && !b2 || !b1 && b2) {
                    int dist = lev.computeLevenshteinDistance(
                                    name1,
                                    name2);
                    if(-1 != dist) {
                        if(null == v1.findEdge(v2)) {
                            Edge e = graph.addEdge(new UndirectedSparseEdge(v1, v2));
                            e.addUserDatum(GraphComparator.SIM_KEY, (double)dist, UserData.SHARED);
                        }
                    }
                }
            }
            
        }
    }    
}
