package connectedsubgraph;

import junit.framework.*;

import matching.UnweightedShortestPathUtil;
import matching.GraphCreator;
import matching.VertexPairSimilar;
import matching.GraphComparator;

import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import edu.uci.ics.jung.utils.UserData;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.decorators.Indexer;
import edu.uci.ics.jung.graph.impl.SparseVertex;

public class DistanceComponentTest extends TestCase {
    
    public DistanceComponentTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(DistanceComponentTest.class);
        
        return suite;
    }

    /**
     * Test of calcShortestDistances method, of class connectedsubgraph.DistanceComponent.
     * Unconnected graphs
         Test data 15                  Test data 25
                
              | 0 Thing                    | 0 Thing
              |                            |
              | 1 Supplier                 | 1 Provider
             /`-.                         /`\ 
           .'    `-.                    .'    \_ 
          /         `-2 Hardware       /        ^--2 Computer
     3 Furniture                     3 Furniture-/ 
                                                      
                 .4 Mobile-Phone                        4 Calculator
                 |                                      
                 .5 Communicator                        5 LED
     *
     * Let's test the graph 15.
     * Vertices set1:   set2:           Shortest paths  Distance
     *          0;      3               (0,1,3)         2
     * 
     */
    public void testCalcShortestDistances() {
        System.out.println("calcShortestDistances");
        //List<DistanceComponent> expResult = null;
        
        // test by GraphCreator.labels15&25, edges15&25 data
        GraphCreator creator = new GraphCreator();
        Graph g15 = creator.createGraph(creator.labels15, creator.edges15);
        UnweightedShortestPath usp = new UnweightedShortestPath(g15);
        //Assert.assertEquals(usp.getDistance((Vertex) id.getVertex(0),(Vertex) id.getVertex(3)).intValue(),2);
        
        Vertex v_thing     = VertexPairSimilar.getVertexByName(g15, "Thing");
        Vertex v_hardware  = VertexPairSimilar.getVertexByName(g15, "Hardware");
        Vertex v_supplier  = VertexPairSimilar.getVertexByName(g15, "Supplier");
        Vertex v_mobile_phone  = VertexPairSimilar.getVertexByName(g15, "Mobile-Phone");
        Set<Vertex> v3 = new HashSet<Vertex>();
        v3.add(v_thing);
        v3.add(v_supplier);
        v3.add(v_hardware);
        
        List<Set<Vertex>> cc = new ArrayList<Set<Vertex>>();
        Set<Vertex> c1 = new HashSet<Vertex>();
        Set<Vertex> c2 = new HashSet<Vertex>();
        c1.add(v_thing);
        c2.add(v_hardware);
        
        // test empty 
        List<DistanceComponent> res = DistanceComponent.calcShortestDistances(cc, usp);
        assertEquals(0, res.size());
        
        // test 2 sets in graph g15
        // assert that shortest path is (0 Thing, 1 Supplier, 2 Hardware)
        cc.add(c1);
        cc.add(c2);
        res = DistanceComponent.calcShortestDistances(cc, usp);
        assertEquals(1, res.size());
        DistanceComponent dc = res.get(0);
        assertEquals(3, dc.path.length);
        for(Vertex v:dc.path) {
            assertTrue(v3.contains(v));
        }
        
        // assert: there is no path from "2 Hardware" to "4 Mobile-Phone"
        c2.clear(); c2.add(v_mobile_phone);
        cc.clear(); cc.add(c1); cc.add(c2);
        res = DistanceComponent.calcShortestDistances(cc, usp);
        assertEquals(0, res.size());
    }

    /**
     * Test of mapSetToNeighbourComponents method, of class connectedsubgraph.DistanceComponent.
     */
    public void testMapSetToNeighbourComponents() {
        System.out.println("mapSetToNeighbourComponents");
        
        List<DistanceComponent> dc = null;
        List<Set<Vertex>> cc = null;
        
        // 1) empty test
        Map<Set<Vertex>, DistanceComponent[]> result = DistanceComponent.mapSetToNeighbourComponents(dc, cc);
        assertEquals(null, result);
        
        // 2) test
        String titles1[] = {"1","2","3"};
        String titles2[] = {"22","33","44"};
        Vertex v;
        
        Set<Vertex> cc1 = new HashSet<Vertex>();
        for(String name:titles1) {
            v = new SparseVertex();
            cc1.add( v );
            v.addUserDatum(GraphComparator.NAME_KEY, name, UserData.SHARED);
        }
        
        Set<Vertex> cc2 = new HashSet<Vertex>();
        for(String name:titles2) {
            v = new SparseVertex();
            cc2.add( v );
            v.addUserDatum(GraphComparator.NAME_KEY, name, UserData.SHARED);
        }
        cc = new ArrayList<Set<Vertex>>();
        cc.add(cc1);
        cc.add(cc2);
        
        dc = new ArrayList<DistanceComponent>();
        DistanceComponent d = new DistanceComponent();
        d.cc1 = cc1;
        d.cc2 = cc2;
        dc.add(d);
        
        result = DistanceComponent.mapSetToNeighbourComponents(dc, cc);
        DistanceComponent[] res_dc = result.get(cc1);
        assertEquals(res_dc.length, 1);
        assertEquals(res_dc[0], d);
        assertEquals(result.get(cc1)[0], result.get(cc2)[0]);
    }
    
}
