package matching;

import junit.framework.*;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.UndirectedGraph;
import edu.uci.ics.jung.graph.decorators.Indexer;
import edu.uci.ics.jung.graph.impl.SparseGraph;
//import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.UndirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.utils.UserData;
import edu.uci.ics.jung.graph.impl.UndirectedSparseEdge;

import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.utils.GraphUtils;

import edu.uci.ics.jung.io.PajekNetReader;
import java.io.IOException;

import java.util.Set;
import java.util.Collection;
import java.util.Map;
import java.util.HashMap;
import java.util.List;

public class GraphComparatorTest extends TestCase {
    
    public GraphComparatorTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(GraphComparatorTest.class);
        
        return suite;
    }

    /** 0.Thing -> 1.Supplier -> 2.Hardware -> 4.Mobile-Phone
     *                  \-> 3.Furniture
     */
    private Graph createTestGraph1() {
        String[] labels = {
            // 0-4 vertices
            "Thing",    // 0
            "Supplier", // 1
            "Hardware", // 2
            "Furniture",// 3
            "Mobile-Phone",// 4
        };
        Graph g = new SparseGraph();
        Vertex[] v = new Vertex[labels.length];
        for (int i = 0; i < labels.length; i++) {
            v[i] = g.addVertex(new SparseVertex());
            v[i].addUserDatum(GraphComparator.NAME_KEY, labels[i], UserData.SHARED);
        }
        // 0-4 first graph
        g.addEdge(new DirectedSparseEdge(v[0], v[1]));
        g.addEdge(new DirectedSparseEdge(v[1], v[2]));
        g.addEdge(new DirectedSparseEdge(v[1], v[3]));
        g.addEdge(new DirectedSparseEdge(v[2], v[4]));
        
        return g;
    }
    
    /** 0.Thing -> 1.Provider -> 2.Computer -> 4.SmartPhone
     *                  \-> 3.Furniture
     */
    private Graph createTestGraph2() {
        String[] labels = {
            // 0-4 vertices
            "Thing",        // 0
            "Provider",     // 1
            "Computer",     // 2
            "Furniture",    // 3
            "Smartphone",   // 4
        };
        Graph g = new SparseGraph();
        Vertex[] v = new Vertex[labels.length];
        for (int i = 0; i < labels.length; i++) {
            v[i] = g.addVertex(new SparseVertex());
            v[i].addUserDatum(GraphComparator.NAME_KEY, labels[i], UserData.SHARED);
        }
        g.addEdge(new DirectedSparseEdge(v[0], v[1]));
        g.addEdge(new DirectedSparseEdge(v[1], v[2]));
        g.addEdge(new DirectedSparseEdge(v[1], v[3]));
        g.addEdge(new DirectedSparseEdge(v[2], v[4]));
        
        return g;
    }
        
    
    public void testCalcNodesStringSimilarity() {
        System.out.println("calcNodesStringSimilarity");
        
        Graph g1 = createTestGraph1();
        Graph g2 = createTestGraph2();
        
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        GraphComparator gc = new GraphComparator();
        
        gc.calcNodesStringSimilarity(g1, g2, lev);
        
        // should be similarities:
        //  graph1       graph2         Levenshtein_distance
        //1 Supplier     Computer       5
        //2 Furniture    Furniture      0
        //3 Thing        Thing          0
        //Set<Vertex>         vert1 = gc.similar_vert.keySet();
        //Collection<Vertex>  vert2 = gc.similar_vert.values();
        
        // create Map <Vertex_Name,Vertex>
        Map<String,Vertex> m = new HashMap<String,Vertex>();
        for(Object o:gc.similar_v1_v2.keySet()) {
            Vertex v = (Vertex)o;
            String s = (String)v.getUserDatum(GraphComparator.NAME_KEY);
            m.put(s, v);
        }
        
        String[] str1 = {"Supplier", "Furniture", "Thing"};
        String[] str2 = {"Computer", "Furniture", "Thing"};
        int[]    dist = { 5,          0,           0};
        for(int i=0;i<dist.length;i++) {
            assertTrue(m.containsKey(str1[i]));
            Vertex v = m.get(str1[i]);
            String s = (String)v.getUserDatum(GraphComparator.NAME_KEY);
            assertEquals(s, str1[i]);
            
            Vertex v2 = gc.similar_v1_v2.get(v);
            String s2 = (String)v2.getUserDatum(GraphComparator.NAME_KEY);
            assertEquals(s2, str2[i]);
            
            int d = (Integer)v.getUserDatum(GraphComparator.SIM_KEY);
            assertEquals(d, dist[i]);
            Vertex v_Supplier = m.get("Supplier");
        }
        
        //testCalcNodesStringSimilarity2();
                
        //testShortestPath();
    }
    
    /** Tests calculation of initial similarity of node names 
     * by GraphCreator.labels3&4, edges3&4 data.
     */
    public void testCalcNodesStringSimilarity2() {
        System.out.println("test 2 calcNodesStringSimilarity");
        
        GraphCreator g = new GraphCreator();
        Graph g1 = g.createGraph(g.labels3, g.edges3);
        Graph g2 = g.createGraph(g.labels4, g.edges4);
        
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        GraphComparator gc = new GraphComparator();
        
        gc.calcNodesStringSimilarity(g1, g2, lev);
        
        assertEquals(3, gc.similar_v1_v2.size());
        
        // create Map <Vertex_Name,Vertex>
        Map<String,Vertex> m = new HashMap<String,Vertex>();
        for(Object o:gc.similar_v1_v2.keySet()) {
            Vertex v = (Vertex)o;
            String s = (String)v.getUserDatum(GraphComparator.NAME_KEY);
            m.put(s, v);
        }
        
        String[] str1 = {"node1", "node2", "node3"};
        String[] str2 = {"node1", "node2", "node3", "node4"};
        int[]    dist = { 0,       0,       0};
        for(int i=0;i<dist.length;i++) {
            assertTrue(m.containsKey(str1[i]));
            Vertex v = m.get(str1[i]);
            String s = (String)v.getUserDatum(GraphComparator.NAME_KEY);
            assertEquals(s, str1[i]);
            
            Vertex v2 = gc.similar_v1_v2.get(v);
            String s2 = (String)v2.getUserDatum(GraphComparator.NAME_KEY);
            assertEquals(s2, str2[i]);
            
            int d = (Integer)v.getUserDatum(GraphComparator.SIM_KEY);
            assertEquals(d, dist[i]);
        }
    }

    
    public void testShortestPath() {
        System.out.println("tesShortestPath");
        UndirectedGraph ug = new UndirectedSparseGraph();
        GraphUtils.addVertices(ug,5);
        Indexer id = Indexer.getIndexer(ug);
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(1));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(1),(Vertex) id.getVertex(2));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(2),(Vertex) id.getVertex(3));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(0),(Vertex) id.getVertex(4));
        GraphUtils.addEdge(ug,(Vertex) id.getVertex(4),(Vertex) id.getVertex(3));

        UnweightedShortestPath usp = new UnweightedShortestPath(ug);
        Assert.assertEquals(usp.getDistance((Vertex) id.getVertex(0),(Vertex) id.getVertex(3)).intValue(),2);
        Assert.assertEquals(((Number) usp.getDistanceMap((Vertex) id.getVertex(0)).get(id.getVertex(3))).intValue(),2);
        Assert.assertNull(usp.getIncomingEdgeMap((Vertex) id.getVertex(0)).get(id.getVertex(0)));
        Assert.assertNotNull(usp.getIncomingEdgeMap((Vertex) id.getVertex(0)).get(id.getVertex(3)));
        
        // Map mIncomingEdgeMap <Vertex, incomingEdge>
        {
            // path from 0 to 3 contains two edges (3,4), (4,0)
            Edge e34 = (Edge)usp.getIncomingEdgeMap((Vertex) id.getVertex(0)).get(id.getVertex(3));
            Vertex v4 = e34.getOpposite((Vertex)id.getVertex(3));
            assertEquals(v4, id.getVertex(4));

            Edge e40 = (Edge)usp.getIncomingEdgeMap((Vertex) id.getVertex(0)).get(v4);
            Vertex v0 = e40.getOpposite(v4);
            assertEquals(v0, id.getVertex(0));
        } 
    }
    
    /**
     * Test of compare method, of class matching.GraphComparator.
     */
    public void testCompare() {
        System.out.println("compare");
        
        // test by GraphCreator.labels1&2, edges1&2 data
        GraphCreator creator = new GraphCreator();
        Graph g1 = creator.createGraph(creator.labels1, creator.edges1);
        Graph g2 = creator.createGraph(creator.labels2, creator.edges2);
        
        // calculates initial similarity
        LevenshteinDistance lev = new LevenshteinDistance();
        lev.use_dist_matrix = true;
        lev.setContstraintType(LevenshteinDistance.TYPE_SOFT);
        
        GraphComparator gc_source = new GraphComparator();
        double r = gc_source.compare(g1, g2, lev);
        assertTrue(VertexPairSimilar.equalsDoubles(r, 0.25));
        
        r = gc_source.compare(g1, null, lev);
        assertTrue(VertexPairSimilar.equalsDoubles(r, 0.0));
        
        r = gc_source.compare(g1, g1, lev);
        assertTrue(VertexPairSimilar.equalsDoubles(r, 1.0));
        
        // test by GraphCreator.labels3&4, edges3&4 data
        g1 = creator.createGraph(creator.labels3, creator.edges3);
        g2 = creator.createGraph(creator.labels4, creator.edges4);
        r = gc_source.compare(g1, g2, lev);
        assertTrue(VertexPairSimilar.equalsDoubles(r, 0.428571429));
        
        String[] fn = {"0romb.net", "1romb.net","2romb.net","3romb.net","4romb.net","5romb.net","6romb.net","7romb.net"};
        String dir = "data/romb/";
        
        Graph g01 = null;
        Graph g02 = null;
        int counter = 0;
        int i=0;
        for(String fn1:fn) {  
            GraphComparator gc = new GraphComparator();
            
            i++;
            int j=0;
            
            for(String fn2:fn) {
                j++;
                
                if(0 != fn1.compareTo(fn2)) {
                    try{
                        PajekNetReader pnr = new PajekNetReader();
                        g01 = pnr.load(dir + fn1);
                        g02 = pnr.load(dir + fn2);
                    }
                    catch (IOException ioe) {
                        System.out.println("TwoModelCompare: unexpected IOException");
                    }
                    
                    GraphComparator.reloadPajekVertexLabelsToGraph(g01);
                    GraphComparator.reloadPajekVertexLabelsToGraph(g02);
                    r = gc.compare(g01, g02, lev);  //- different ?+ 0.2857142857142857
                    //r = gc.compare(g1, g2, lev);  //+ 0.42857142857142855
                    counter ++;
                    
                    /*System.out.println("fn1="+fn1+"; vertices="+g01.numVertices()+"; edges="+g01.numEdges());
                    System.out.println("fn1="+fn2+"; vertices="+g02.numVertices()+"; edges="+g02.numEdges());
                    System.out.println("   sim="+r);*/
                    System.out.println(" " + i + " " + j + " " + r);
                }
            }
        }
        System.out.println(" == counter="+counter);
    }
}
